﻿#region Copyright
// --------------------------------------------------------------------------------------------------------------------
// <copyright file="EnumDomain.cs" company="Ian Horswill">
// Copyright (C) 2018 Ian Horswill
//  
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in the
// Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
// and to permit persons to whom the Software is furnished to do so, subject to the
// following conditions:
//  
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </copyright>
// --------------------------------------------------------------------------------------------------------------------
#endregion
using System.Diagnostics;

namespace CatSAT
{
    /// <summary>
    /// A domain whose values are the values of an enumerated type.
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class EnumDomain<T> : FDomain<T>
    {
        private EnumDomain() : base(typeof(T).Name, (T[]) typeof(T).GetEnumValues())
        {
            Debug.Assert(typeof(T).IsEnum);
        }

        /// <summary>
        /// The unique EnumDomain for enum type T.
        /// </summary>
        public static readonly EnumDomain<T> Singleton = new EnumDomain<T>();
    }
}
